package envoy.client;

import java.awt.Color;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.prefs.Preferences;

import envoy.client.ui.Theme;

/**
 * Project: <strong>envoy-client</strong><br>
 * File: <strong>Settings.java</strong><br>
 * Created: <strong>11 Nov 2019</strong><br>
 * 
 * @author Leon Hofmeister
 * @author Maximilian K&auml;fer
 * @author Kai S. K. Engelbart
 * @since Envoy v0.2-alpha
 */
public class Settings {

	// Actual settings accessible by the rest of the application
	private boolean				enterToSend	= true;
	private Map<String, Theme>	themes;
	private String				currentTheme;

	/**
	 * Required to save the settings.
	 */
	private Preferences prefs = Preferences.userNodeForPackage(Settings.class);

	/**
	 * User-defined themes are stored inside this file.
	 */
	private File themeFile = new File("themes.ser");

	/**
	 * Singleton instance of this class.
	 */
	private static Settings settings = new Settings();

	/**
	 * The way to instantiate the settings.
	 * Is set to private to deny other instances of that object.
	 * 
	 * @since Envoy v0.2-alpha
	 */
	private Settings() { load(); }

	/**
	 * This method is used to ensure that there is only one instance of Settings.
	 * 
	 * @return the instance of Settings
	 * @since Envoy v0.2-alpha
	 */
	public static Settings getInstance() { return settings; }

	@SuppressWarnings("unchecked")
	private void load() {
		setEnterToSend(prefs.getBoolean("enterToSend", true));
		setCurrentTheme(prefs.get("theme", "dark"));

		// Load themes from theme file
		try (ObjectInputStream in = new ObjectInputStream(new FileInputStream(themeFile))) {
			Object obj = in.readObject();
			if (obj instanceof HashMap) themes = (Map<String, Theme>) obj;
		} catch (IOException | ClassNotFoundException e) {
			themes			= new HashMap<>();
			currentTheme	= "dark";
			e.printStackTrace();
		}

		// Load standard themes not defined in the themes file
		themes.put("dark",
				new Theme("dark", Color.black, Color.darkGray, Color.white, Color.blue, Color.white, Color.orange, Color.blue, Color.white,
						Color.white));
		themes.put("light",
				new Theme("light", new Color(235, 235, 235), Color.white, Color.white, Color.darkGray, Color.black, Color.orange, Color.darkGray,
						Color.black, Color.black));
	}

	/**
	 * Updates the preferences when the save button is clicked.
	 * 
	 * @throws IOException if something went wrong while saving
	 * @since Envoy v0.2-alpha
	 */
	public void save() throws IOException {
		prefs.put("theme", currentTheme);
		prefs.putBoolean("enterToSend", isEnterToSend());

		// Save themes to theme file
		themeFile.createNewFile();
		try (ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream(themeFile))) {
			out.writeObject(themes);
		}
	}

	/**
	 * adds new theme to the theme map and sets current theme to the new theme.
	 * 
	 * @param theme the theme to add
	 * @since Envoy v0.2-alpha
	 */
	public void addNewThemeToMap(Theme theme) {
		settings.getThemes().put(theme.getThemeName(), theme);
		// currentTheme = theme.getThemeName();
	}

	/**
	 * @return the name of the current theme
	 * @since Envoy v0.2-alpha
	 */
	public String getCurrentTheme() { return currentTheme; }

	/**
	 * Sets the currentTheme
	 * 
	 * @param themeName the name of the new current theme
	 * @since Envoy v0.2-alpha
	 */
	public void setCurrentTheme(String themeName) { currentTheme = themeName; }

	/**
	 * @return true, if "enter" suffices to send a message, else it has to be "ctrl"
	 *         + "enter"
	 * @since Envoy v0.2-alpha
	 */
	public boolean isEnterToSend() { return enterToSend; }

	/**
	 * Change mode of posting a message via Keystroke.
	 * 
	 * @param enterToSend if true, "enter" suffices to send a message, <br>
	 *                    else it has to be "ctrl" + "enter"
	 * @since Envoy v0.2-alpha
	 */
	public void setEnterToSend(boolean enterToSend) { this.enterToSend = enterToSend; }

	/**
	 * @return the map of all themes 
	 * @since Envoy v0.2-alpha
	 */
	public Map<String, Theme> getThemes() { return themes; }

	/**
	 * Sets the map of all themes
	 * 
	 * @param themes a map(String, Theme) of all themes
	 * @since Envoy v0.2-alpha
	 */
	public void setThemes(Map<String, Theme> themes) { this.themes = themes; }
}